@@ -16,7 +16,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::XMP;
 
-$VERSION = '1.11';
+$VERSION = '1.13';
 
 sub ProcessXtra($$$);
 
@@ -88,8 +88,8 @@ sub ProcessXtra($$$);
     Rating => {
         Name => 'RatingPercent',
         Notes => q{
-            normal Rating values of 1,2,3,4 and 5 stars correspond to RatingPercent
-            values of 1,25,50,75 and 99 respectively
+            called Rating by the spec.  XMP-xmp:Rating values of 1,2,3,4 and 5 stars
+            correspond to RatingPercent values of 1,25,50,75 and 99 respectively
         },
     },
 );
@@ -445,7 +445,7 @@ my %sRegions = (
     '{F29F85E0-4FF9-1068-AB91-08002B27B3D9} 6'     => 'Comments',
     '{64440492-4C8B-11D1-8B70-080036B11A03} 11'    => {
         Name => 'Copyright',
-        Groups => { 2 => 'Copyright' },
+        Groups => { 2 => 'Author' },
     },
     '{56A3372E-CE9C-11D2-9F0E-006097C686F6} 2'     => 'Artist',
     '{56A3372E-CE9C-11D2-9F0E-006097C686F6} 4'     => 'AlbumTitle',
